import 'dart:convert';
import 'package:datadome_flutter_dio/datadome_dio.dart';
import 'package:datadome_flutter_dio/datadome_logger.dart';
import 'package:datadome_flutter_dio/tracking/event_data.dart';
import 'package:datadome_flutter_dio/tracking/event_tracker.dart';
import 'package:datadome_flutter_dio/retriable_request.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'captcha_page.dart';

typedef OnCaptchaDisplay = void Function(Widget view);
typedef OnCaptchaDismiss = void Function();

class DataDomeInterceptor extends Interceptor {
  String? dataDomeKey;
  Dio? dio;
  late BuildContext? context;
  static List<RetriableRequest> failedRequests = [];
  static Map<String, dynamic> deviceInfos = <String, dynamic>{};
  late OnCaptchaDisplay? onCaptchaDisplay;
  late OnCaptchaDismiss? onCaptchaDismiss;

  // Storing DataDome client key and application context for payload construction
  static Map<String, dynamic> payloadInitializer = <String, dynamic>{};

  DataDomeInterceptor(String dataDomeKey, Dio dio, BuildContext context) {
    this.dataDomeKey = dataDomeKey;
    this.dio = dio;
    this.context = context;
    this.onCaptchaDisplay = null;
    this.onCaptchaDismiss = null;
    payloadInitializer["context"] = context;
    payloadInitializer["dataDomeKey"] = dataDomeKey;
  }

  DataDomeInterceptor.withCallback(String dataDomeKey, Dio dio, OnCaptchaDisplay onCaptchaDisplay, OnCaptchaDismiss onCaptchaDismiss) {
    this.dataDomeKey = dataDomeKey;
    this.dio = dio;
    this.context = null;
    this.onCaptchaDisplay = onCaptchaDisplay;
    this.onCaptchaDismiss = onCaptchaDismiss;
  }

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) async {
    DataDomeLogger.info('Validating request ${options.uri}');

    String cookie = await getDataDomeCookie()?? '';
    if (cookie.isNotEmpty) {
      options.headers['Cookie'] = cookie;
    }

    await fillPayload(cookie, options);

    return handler.next(options);
  }

  Future<void> fillPayload(String cookie, RequestOptions options) async {
    deviceInfos = await EventTracker().initPlatformState();
    deviceInfos["ua"] = options.headers["User-Agent"];
    deviceInfos["ddk"] = dataDomeKey;//client key
    deviceInfos["cid"] = DataDomeDio.parseCookieValue(cookie);
    deviceInfos["request"] = options.uri;

    if(context != null) {
      // complete device infos of payload that requires context
      MediaQueryData? media = MediaQuery.maybeOf(context!);
      if(media != null) {
        // The context does have a MediaQuery
        deviceInfos["screen_x"] = media.size.width.toStringAsFixed(3);
        deviceInfos["screen_y"] = media.size.height.toStringAsFixed(3);
        deviceInfos["screen_d"] = media.devicePixelRatio;
      }
    }
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    super.onResponse(response, handler);
    EventTracker().sendEventPayload();
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    EventTracker().addEvent(getEvent(Events.RESPONSE_VALIDATION, EventTracker.sourceSdk));

    if(err.response != null) {
      var ddHeaders = err.response?.headers.map.containsKey('x-dd-b');
      var code = err.response!.statusCode;
      if ((code == 401 || code == 403) && (ddHeaders != null && ddHeaders)) {
        //blocked by datadome
        failedRequests.add(RetriableRequest(dio, err, handler));
        if (CaptchaPage.isDisplayed) {
          return;
        }

        if(err.response == null) {
          DataDomeLogger.error("Null DioError response info");
          return;
        }

        var captchaUrl = "";
        var response = err.response;
        switch(response!.requestOptions.responseType) {
          case ResponseType.json:
            captchaUrl = response.data['url'];
            break;
          case ResponseType.stream:
            DataDomeLogger.warning("The SDK does not support Stream ResponseType. Please change it to json.");
            break;
          case ResponseType.plain:
            try {
              var jsonObj = jsonDecode(response.data);
              captchaUrl = jsonObj['url'];
            } catch(e) {
              DataDomeLogger.error("Error decoding the response body to retrieve URL: $e");
              captchaUrl = ''; // Return an empty URL value
            }
            break;
          case ResponseType.bytes:
            try {
              final decodedJSON = jsonDecode(utf8.decode(response.data));
              captchaUrl = decodedJSON['url'];
            } catch (e) {
              DataDomeLogger.error("Error decoding the response body to retrieve URL: $e");
              captchaUrl = ''; // Return an empty URL value
            }
            break;
        }

        if(captchaUrl.trim().isEmpty) {
          DataDomeLogger.warning("Missing or empty captcha URL in response body");
          return;
        }

        if (context != null) {
          showCaptcha(
              context!,
              captchaUrl,
                  (String cookie) {
                didResolveCaptcha(cookie);
              }
          );
        } else {
          Widget widget = new CaptchaPage(captchaUrl: captchaUrl,
              onCaptchaResolved: (String cookie) {
                didResolveCaptcha(cookie);
              });
          if (onCaptchaDisplay != null) {
            onCaptchaDisplay!(widget);
          }
        }
        return;
      }
    }

    super.onError(err, handler);
  }

  late BuildContext dialogContext;
  void showCaptcha(BuildContext context, String? captchaUrl, ValueChanged<String> onCaptchaResolved) {
    DataDomeLogger.info('Presenting the captcha');

    CaptchaPage.isDisplayed = true;
    showGeneralDialog(
      context: context,
      barrierDismissible: false,
      pageBuilder: (context, __, ___) {
        dialogContext = context;
        return new CaptchaPage(captchaUrl: captchaUrl, onCaptchaResolved: (value){
          onCaptchaResolved(value);
          DataDomeDio.storeCookie(value);
          dismissCaptcha();
        });
      },
    );
  }

  void dismissCaptcha() {
    DataDomeLogger.info('Dismissing the captcha');

    if(onCaptchaDismiss != null) {
      onCaptchaDismiss!();
    } else if(CaptchaPage.isDisplayed) {
      Navigator.pop(dialogContext);
    }
    CaptchaPage.isDisplayed = false;
  }

  void didResolveCaptcha(String cookie) {
    DataDomeLogger.info("Did resolved captcha with cookie $cookie");

    DataDomeDio.storeCookie(cookie);
    dismissCaptcha();

    DataDomeLogger.info("Retrying all failed requests");
    retryFailedRequests();
  }

  void retryFailedRequests() {
    for (var retriable in failedRequests) {
      retry(retriable);
    }

    failedRequests.clear();
  }

  Future<Null> retry(RetriableRequest retriable) async {
    RequestOptions requestOptions = retriable.error.requestOptions;
    final options = new Options(
      method: requestOptions.method,
      headers: requestOptions.headers,
    );

    Response response = await retriable.dio!.request(
      requestOptions.path,
      cancelToken: requestOptions.cancelToken,
      data: requestOptions.data,
      onReceiveProgress: requestOptions.onReceiveProgress,
      onSendProgress: requestOptions.onSendProgress,
      queryParameters: requestOptions.queryParameters,
      options: options,
    );

    retriable.handler.resolve(response);
  }

  /// Returns the DataDome cookie. The returned value should be a string representing the cookie with the datadome name and its value.
  /// Example: "datadome=xxxx"
  Future<String> getDataDomeCookie() async {
    return DataDomeDio.getCookie();
  }

  /// Saves a DataDome cookie. The new value should be a string representing the cookie with its value and its attributes.
  /// Example: "datadome=xxxx; Max-Age=31536000; Domain=.yourdomain.com; Path=/; Secure; SameSite=Lax"
  void setDataDomeCookie(String newValue) {
    DataDomeDio.storeCookie(newValue);
  }
}
