import 'package:datadome_flutter_dio/datadome_logger.dart';
import 'package:datadome_flutter_dio/tracking/event_data.dart';
import 'package:dio/dio.dart';
import 'package:flutter/services.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'dart:io';

import '../datadome_dio.dart';

class EventTracker {

  static final EventTracker sharedInstance = EventTracker._internal();

  factory EventTracker() {
    return sharedInstance;
  }

  EventTracker._internal();

  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();
  static Map<String, dynamic> _deviceData = <String, dynamic>{};
  List<EventData> events = <EventData>[];
  static final sourceSdk = "sdk";
  var lastTrackRequest = 0;
  final trackRequestInterval = 10000;
  final minimumEventThreshold = 5;
  var isSendingTrackingData = false;

  Future<Map<String, dynamic>> initPlatformState() async {
    var deviceData = <String, dynamic>{};

    try {
      if (Platform.isAndroid) {
        deviceData =
            androidDeviceInfos(await deviceInfoPlugin.androidInfo);
      } else if (Platform.isIOS) {
        deviceData = iosDeviceInfos(await deviceInfoPlugin.iosInfo);
      }
    } on PlatformException {
      deviceData = <String, dynamic>{
        'Error:': 'Failed to get platform version.'
      };
    }

    deviceData["ddvc"] = await getAppVersion();//applicationVersion
    //to be updated with release
    deviceData["ddv"] = "2.1.1"; //SDK version
    deviceData["inte"] = "Flutter-Dio";

    _deviceData = deviceData;
    return deviceData;
  }

  Future<String> getAppVersion() async{
    var pkData = await PackageInfo.fromPlatform();
    return pkData.version;
  }

  Map<String, dynamic> androidDeviceInfos(AndroidDeviceInfo build) {
    return <String, dynamic> {
        "os": "Android",
        "osr": build.version.release,
        "osn": build.version.codename,
        "osv": build.version.sdkInt,
        "events": events.map((event) => event.customJsonString()),
        "mdl": build.model,
        "prd": build.product,
        "mnf": build.manufacturer,
        "dev": build.device,
        "hrd": build.hardware,
        "fgp": build.fingerprint,
        "tgs": build.tags,
    };
  }

  Map<String, dynamic> iosDeviceInfos(IosDeviceInfo data) {
    String? osVersion = data.systemVersion;
    if(data.systemVersion != null) {
      //Split parts with '.' delimiter
      List<String> resultingOSV = data.systemVersion!.split(".");
      if(resultingOSV.length > 2) {
        osVersion = resultingOSV[0]+"."+resultingOSV[1];
      }
    }

    return <String, dynamic> {
      "os": "iOS",
      "osr": data.systemVersion,
      "osn": data.systemName,
      "osv": osVersion,
      "events": events.map((event) => event.customJsonString()),
      "mdl": data.name,
    };
  }

  void addEvent(EventData event) {
    if(events.length <= 80) {
      events.add(event);
    }
  }

  static String eventPositionTemplate(double x, double y) {
    return "{\"x\":${x.toStringAsFixed(3)}, \"y\":${y.toStringAsFixed(3)}}";
  }

  Future<Null> sendEventPayload() async {
    if(lastTrackRequest > 0 && events.length <= minimumEventThreshold) {
      return null;
    }
    if(!isSendingTrackingData && (DateTime.now().millisecondsSinceEpoch - lastTrackRequest > trackRequestInterval)) {
      isSendingTrackingData = true;
      DataDomeLogger.info("Logging events");

      var dio = Dio();
      var url = "https://api-sdk.datadome.co/sdk/";

      var params = _deviceData;
      Response response = await dio.post(url,
        options: Options(headers: {
          "Content-Type": "application/x-www-form-urlencoded",
        }),
        data: params,
      );

      // Parsing event tracking response and store new cookie
      if(response.statusCode == 200) {
        if(response.data != null && response.data['cookie'] != null && (response.data['cookie'] as String).isNotEmpty) {
          var responseBody = EventTrackingResponse.fromJson(response.data);
          DataDomeDio.storeCookie(responseBody.cookie);
        }
      }

      // After sending payload
      isSendingTrackingData = false;
      lastTrackRequest = DateTime.now().millisecondsSinceEpoch;
      events.clear();
    }
  }

  // setDeviceData updates _deviceData map with collected data without integrating DataDomeInterceptor
  set setDeviceData(Map<String, dynamic> deviceData) {
    _deviceData = deviceData;
  }
}

class EventTrackingResponse {
  EventTrackingResponse({required this.status, required this.cookie});
  final int status; // non-nullable
  final String cookie; // non-nullable

  factory EventTrackingResponse.fromJson(Map<String, dynamic> data) {
    final status = data['status'] as int; // cast as non-nullable int
    final cookie = data['cookie'] as String; // cast as non-nullable String
    return EventTrackingResponse(status: status, cookie: cookie);
  }
}
